--[[ *** LOAD LIBRARIES *** 

	Force used libraries to load. This is necessary because load on demand addons (in our case,
	disembedded libs) will no longer load even when listed in OptionalDeps and can only be loaded
	via LoadAddOn(). This must be done before a LibStub("AceAddon-3.0"):NewAddon() call due to
	ADDON_LOADED firing, as AceAddon-3.0 will call :OnInit() on any addon's ADDON_LOADED event due
	to needing to support submodules which may not be its their own addons.
]]--
LoadAddOn("Ace3")
LoadAddOn("LibDataBroker-1.1")
LoadAddOn("LibDBIcon-1.0")
LoadAddOn("LibDataBroker-1.1")
LoadAddOn("LibQTip-1.0")

--[[ *** ADDON DECLARATION *** ]]--
local modName = "Guilder"
local _G = _G
local Guilder = LibStub("AceAddon-3.0"):NewAddon("Guilder", "AceComm-3.0", "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0")
_G.Guilder = Guilder

--[[ *** DEFINE LIBRARIES *** ]]--
local ACD3 = LibStub("AceConfigDialog-3.0")
local AL = LibStub("AceLocale-3.0"):GetLocale("Guilder")
Guilder.AL = AL
local CTL = assert(ChatThrottleLib, "Guilder requires ChatThrottleLib")
local LDB = LibStub("LibDataBroker-1.1", true)
local LDBIcon = LDB and LibStub("LibDBIcon-1.0", true)
LG = LibStub("LibGuild-1.0", true)
if not LG then
	Guilder:Print("LibGuild-1.0 not found!")
	return
end
local QTC = LibStub("LibQTip-1.0")

--[[ *** SET UP VARIABLES *** ]]--

-- Make global API calls local to speed things up
local pairs = _G.pairs
local string = _G.string
local table = _G.table
local tinsert = _G.tinsert

local CreateFrame = _G.CreateFrame
local GetFriendInfo = _G.GetFriendInfo
local GetNumFriends = _G.GetNumFriends
local IsInGuild = _G.IsInGuild
local UIParent = _G.UIParent

-- Colours stored for code readability
RESET = "|r"
BLIZZARD = "|cFF00C0FF"
BLUE = "|cFF0070DD"
CYAN = "|cFF00C0FF"
GREEN = "|cFF1EFF00"
GREY = "|cFF999999"
ORANGE = "|cFFFF8400"
PURPLE = "|cFF9F3FFF"
RED = "|cFFFF0000"
WHITE = "|cFFFFFFFF"
YELLOW = "|cFFFFFF00"

-- Create an LDB object
if LDB then
	LDBObj = LDB:NewDataObject("Guilder", {
		type = "launcher",
		label = "Guilder",
		text = "Guilder",
		icon = "Interface\\AddOns\\Guilder\\Images\\icon.tga"
	})
end

-- if the LibDBIcon library is available, create a minimap icon
local f = CreateFrame("Frame")
f:SetScript("OnEvent", function()
	if LDBIcon then
		LDBIcon:Register("Guilder", LDBObj, Guilder.db.profile.minimap)
	end
	f:UnregisterEvent("PLAYER_LOGIN")
end)
f:RegisterEvent("PLAYER_LOGIN")

-- Guilder variables
msgFrame = nil
msgFrameCreated = "0"
myName = UnitName("player")
myFaction, _ = UnitFactionGroup("player")
myNameFaction = myName.." - "..myFaction
myVersionNumber = GetAddOnMetadata("Guilder", "Version")
local gldrTooltip = nil

--[[ *** DEFAULT SETTINGS *** ]]--

local defaults = {
	profile = {
		info = {
			showOnline = false,
			showTotal = false,
			showOffline = true,
			showModules = false,
			showHint = true,
		},
		minimap = {
			hide = false,
			minimapPos = 90,
			radius = 80,
		},
		modules = {
			["Busy Responder"] = false,
			["Crowd Control"] = false,
		},
		tooltip = {},
		dbCache = {},
	},
}

--[[ *** CONFIGURATION MENU *** ]]--

local options = {
	type = "group",
	args = {
		defaultArgs = {
			type = "group",
			name = AL["Guilder"],
			args = {
				header = {
					type = "header",
					name = GetAddOnMetadata("Guilder", "Title"),
					order = 9,
				},
				Info = {
					type = "description",
					name = GetAddOnMetadata("Guilder", "Notes"),
					order = 10,
				},
				settingsHeader = {
					type = "header",
					name = AL["Settings"],
					order = 11,
				},
				aceConfig = {
					type = "execute",
					name = AL["Standalone Config"],
					desc = AL["Open a standalone config window. You might consider installing |cffffff00BetterBlizzOptions|r to make the Blizzard UI options panel resizable."],
					order = 12,
					func = function()
						InterfaceOptionsFrame:Hide()
						ACD3:SetDefaultSize("Guilder", 500, 550)
						ACD3:Open("Guilder")
					end,
				},
				info = {
					type = "group",
					name = AL["Information"],
					order = 20,
					guiInline = true,
					args = {
						Online = {
							type = "toggle",
							name = AL["Show Online"],
							desc = AL["Show number of officers online"],
							order = 1,
							get = function() return Guilder.db.profile.info.showOnline end,
							set = function() Guilder.db.profile.info.showOnline = not Guilder.db.profile.info.showOnline; Guilder:UpdateText() end,
						},
						Total = {
							type = "toggle",
							name = AL["Show Total"],
							desc = AL["Show total number of officers"],
							order = 2,
							get = function() return Guilder.db.profile.info.showTotal end,
							set = function() Guilder.db.profile.info.showTotal = not Guilder.db.profile.info.showTotal; Guilder:UpdateText() end,
						},
						Offline = {
							type = "toggle",
							name = AL["Show Offline Officers"],
							desc = AL["Show offline officers in tooltip"],
							order = 3,
							get = function() return Guilder.db.profile.info.showOffline end,
							set = function() Guilder.db.profile.info.showOffline = not Guilder.db.profile.info.showOffline end,
						},
						Modules = {
							type = "toggle",
							name = AL["Show Module State"],
							desc = AL["Show module state in tooltip"],
							order = 4,
							get = function() return Guilder.db.profile.info.showModules end,
							set = function() Guilder.db.profile.info.showModules = not Guilder.db.profile.info.showModules end,
						},
						Hint = {
							type = "toggle",
							name = AL["Show Hint"],
							desc = AL["Show hint at the bottom of the tooltip"],
							order = 5,
							get = function() return Guilder.db.profile.info.showHint end,
							set = function() Guilder.db.profile.info.showHint = not Guilder.db.profile.info.showHint end,
						},
						Minimap = {
							type = "toggle",
							name = AL["Hide Minimap Icon"],
							desc = AL["Hides the minimap icon"],
							order = 6,
							get = function () return Guilder.db.profile.minimap.hide end,
							set = function (info, v)
								Guilder.db.profile.minimap.hide = v
								if v then
									LDBIcon:Hide("Guilder")
								else
									LDBIcon:Show("Guilder")
								end
							end,
						}
					},
				},
				ranks = {
					type = "group",
					name = AL["Ranks"],
					order = 21,
					guiInline = true,
					args = {
						info = {
							type = "description",
							name = AL["You are not in a guild!"],
						},
					},
				},
				rescan = {
					type = "group",
					name = AL["Miscellaneous"],
					order = 22,
					guiInline = true,
					args = {
						info = {
							type = "description",
							name = AL["Things don't always go as planned. If you can't see the guild ranks and you are definitely in a guild, please use the rescan button."],
							order = 1,
						},
						rescan = {
							type = "execute",
							name = AL["Rescan"],
							order = 2,
							func = function() Guilder:CreateRanks() end,
						},
					}
				}
			},
		},
		config = {
			type = "execute",
			guiHidden = true,
			name = AL["Configure"],
			desc = AL["Configure"],
			func = Guilder.OpenConfig
		},
		modules = {
			type = "group",
			name = AL["Modules"],
			desc = AL["Modules"],
			args = {}
		},
	},
}

--[[ *** EVENT FUNCTIONS *** ]]--

Guilder:SetDefaultModuleState(false)

do
	local timer, t = nil, 0
	local function update()
		t = t + arg1
		if t > 0.5 then
			timer:SetScript("OnUpdate", nil)
			Guilder:UpdateConfig()
		end
	end
	function Guilder:SetUpdateConfig()
		t = 0
		timer = timer or CreateFrame("Frame", nil, UIParent)
		timer:SetScript("OnUpdate", update)
	end
end

function Guilder:OnDisable()
--[[
	This function is part of the provided API
]]
	LG:UnregisterAll()
end

function Guilder:OnEnable()
--[[
	This function is part of the provided API
]]
	self:CreateMsgFrame()
	Guilder:Print(AL["Welcome to Guilder! Type /guilder or /gldr to configure."])
	for k, v in self:IterateModules() do
		if Guilder.db.profile.modules[k] ~= false then
			v:Enable()
		end
	end

	if IsInGuild() then
		LG:RegisterCallback("Update", function() Guilder:UpdateText() end)
	end
	-- Catch whispers and filter what should be shown
	self:RawHook("ChatFrame_MessageEventHandler", "WhisperHandler", true)
	self:RegisterEvent("CHAT_MSG_WHISPER")
end

function Guilder:OnInitialize()
--[[
	This function is part of the provided API
]]
	-- Register database
	local db = LibStub("AceDB-3.0"):New("GuilderDB", defaults, "Default")
	self.db = db
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("Guilder", options)
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("GuilderModules", options.args.modules)
	self.optionsFrame = ACD3:AddToBlizOptions("Guilder", GetAddOnMetadata("Guilder", "Title"), nil, "defaultArgs")

	-- Fill up our modular options...
	for k, v in self:IterateModules() do
		options.args.modules.args[k:gsub(" ", "_")] = {
			type = "group",
			name = (v.modName or k),
			args = nil
		}
		local t
		if v.GetOptions then
			t = v:GetOptions()
			t.settingsHeader = {
				type = "header",
				name = AL["Settings"],
				order = 12
			}
		end
		t = t or {}
		t.toggle = {
			type = "toggle", 
			name = v.toggleLabel or (AL["Enable "] .. (v.modName or k)), 
			width = "double",
			desc = v.Info and v:Info() or (AL["Enable "] .. (v.modName or k)), 
			order = 11,
			get = function()
				return Guilder.db.profile.modules[k] ~= false or false
			end,
			set = function(info, v)
				Guilder.db.profile.modules[k] = v
				if v then
					Guilder:EnableModule(k)
					Guilder:Print(AL["Enabled"], k, AL["Module"])
				else
					Guilder:DisableModule(k)
					Guilder:Print(AL["Disabled"], k, AL["Module"])
				end
			end
		}
		t.header = {
			type = "header",
			name = v.modName or k,
			order = 9
		}
		if v.Info then
			t.description = {
				type = "description",
				name = v:Info() .. "\n\n",
				order = 10
			}
		end
		options.args.modules.args[k:gsub(" ", "_")].args = t
	end

	local moduleList = {}
	local moduleNames = {}
	for k, v in pairs(options.args.modules.args) do
		moduleList[v.name] = k
		tinsert(moduleNames, v.name)
	end
	table.sort(moduleNames)
	for _, name in ipairs(moduleNames) do
		ACD3:AddToBlizOptions("GuilderModules", name, "Guilder", moduleList[name])
	end

	if not options.args.Profiles then
 		options.args.Profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(db)
		self.optionsFrame.Profiles = ACD3:AddToBlizOptions("Guilder", AL["Profiles"], "Guilder", "Profiles")
		-- Add in the about panel to the Bliz options (but not the ace3 config)
		if LibStub:GetLibrary("LibAboutPanel", true) then
			self.optionsFrame["About"] = LibStub:GetLibrary("LibAboutPanel").new("Guilder", "Guilder")
		else
			Guilder:Print(RED.."LibAboutPanel not loaded.")
		end
	end

	-- Register slash commands
	self:RegisterChatCommand("gldr", "OpenConfig")
	self:RegisterChatCommand("guilder", "OpenConfig")

	-- hook Profile functions -- not used atm
	-- db.RegisterCallback(self, "OnProfileChanged", "SetUpdateConfig")
	-- db.RegisterCallback(self, "OnProfileCopied", "SetUpdateConfig")
	-- db.RegisterCallback(self, "OnProfileReset", "SetUpdateConfig")

	-- first time this character is using the addon, so create rank tables
	if db.profile.dbCache[myNameFaction] == nil then
		db.profile.dbCache[myNameFaction] = {}
		db.profile.dbCache[myNameFaction].ranknames = {}
		db.profile.dbCache[myNameFaction].ranks = {}
		for i = 1, 10 do
			db.profile.dbCache[myNameFaction].ranknames[i] = ""
			db.profile.dbCache[myNameFaction].ranks[i] = "3-Member"
		end
	end

	-- create alias for cached information
	dbCache = db.profile.dbCache[myNameFaction]

	if IsInGuild then
		GuildRoster()
		self:CreateRanks(dbCache.ranknames[1] ~= "")
	end
	-- Update the ranks once the addon has been loaded
	self:RegisterEvent("GUILD_ROSTER_UPDATE")
	-- Update the ranks selection is the player leaves / joins a guild
	self:RegisterEvent("PLAYER_GUILD_UPDATE")
end

--[[ *** CREATE MESSAGE FRAME *** ]]--

-- This function creates a frame to use as a popup window to share information
-- @param nil
-- @usage internal use only
-- @return nil
function Guilder:CreateMsgFrame()
--[[
	This function creates a frame to use as a popup window to share information

	This function was adapted from the similar function provided in the ACE library API for
	creating an 'About' information window for a FuBar compatible plugin mod.
]]
	-- Create the window frame object and configure its properties.
	msgFrame = CreateFrame("Frame", "Acemod30msgFrame", UIParent, "DialogBoxFrame")
	msgFrame:SetWidth(500)
	msgFrame:SetHeight(400)
	msgFrame:SetPoint("CENTER")
	msgFrame:SetBackdrop({
		bgFile = [[Interface\DialogFrame\UI-DialogBox-Background]],
		edgeFile = [[Interface\Tooltips\UI-Tooltip-Border]],
		tile = true, tileSize = 16, edgeSize = 16,
		insets = { left = 5, right = 5, top = 5, bottom = 5 }
	})
	msgFrame:SetBackdropColor(0,0,0,1)

	-- Set the window's title text.
	local text = msgFrame:CreateFontString(nil, "OVERLAY", "GameFontHighlightLarge")
	msgFrame.title = text
	text:SetPoint("TOP", 0, -5)

	-- Initialise the window's text arrays.
	msgFrame.lefts = {}
	msgFrame.rights = {}
	msgFrame.textLefts = {}
	msgFrame.textRights = {}

	-- Object function: Clear the window.
	function msgFrame:Clear()
		self.title:SetText("")
		for i = 1, #self.lefts do
			self.lefts[i] = nil
			self.rights[i] = nil
		end
	end

	-- Object function: Prepare the window for a new version check.
	function msgFrame:PrepareForVersionCheck()
		self:Clear()
		msgFrame.title:SetText("Guilder")
		msgFrame:AddLine(AL["Player"], AL["Version"])
		msgFrame:AddLine("@NULL@", " ")
		msgFrame:AddLine(myName, BLIZZARD..myVersionNumber)
	end

	-- Object function: Add a line of text to the window.
	function msgFrame:AddLine(left, right)
		msgFrame.lefts[#msgFrame.lefts+1] = left
		msgFrame.rights[#msgFrame.rights+1] = right
	end

	-- Object function: Show the window.
	local msgFrame_Show = msgFrame.Show
	function msgFrame:Show(...)
		local maxLeftWidth = 0
		local maxRightWidth = 0
		local textHeight = 0
		for i = 1, #self.lefts do
			if not self.textLefts[i] then
				local left = msgFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
				self.textLefts[i] = left
				local right = msgFrame:CreateFontString(nil, "OVERLAY", "GameFontHighlight")
				self.textRights[i] = right
				if i == 1 then
					left:SetPoint("TOPRIGHT", msgFrame, "TOPLEFT", 75, -35)
				else
					left:SetPoint("TOPRIGHT", self.textLefts[i-1], "BOTTOMRIGHT", 0, -5)
				end
				right:SetPoint("LEFT", left, "RIGHT", 5, 0)
			end
			if self.lefts[i] == "@NULL@" then
				self.textLefts[i]:SetText(" ")
			else
				self.textLefts[i]:SetText(self.lefts[i] .. ":")
			end

			self.textRights[i]:SetText(self.rights[i])
			local leftWidth = self.textLefts[i]:GetWidth()
			local rightWidth = self.textRights[i]:GetWidth()
			textHeight = self.textLefts[i]:GetHeight()
			if maxLeftWidth < leftWidth then
				maxLeftWidth = leftWidth
			end
			if maxRightWidth < rightWidth then
				maxRightWidth = rightWidth
			end
		end
		for i = #self.lefts+1, #self.textLefts do
			self.textLefts[i]:SetText('')
			self.textRights[i]:SetText('')
		end
		msgFrame:SetWidth(75 + maxRightWidth + 20)
		msgFrame:SetHeight(#self.lefts * (textHeight + 5) + 100)

		msgFrame_Show(self, ...)
	end
	-- Set a flag indicating that the window has been created.
	msgFrameCreated = "1"
end

--[[ *** GENERIC FUNCTIONS *** ]]--

function Guilder:GetArgs(message, separator)
--[[
	This function receives a string and converts it into a list of arguments, using "separator" as
	the separator character.
]]
	if message == nil then
		message = ""
	end

	local args = {}
	local i = 0
	-- search for seperators in the string and return the separated data
	for value in string.gmatch(message, "[^"..separator.."]+") do
		i = i + 1
		args[i] = value
	end
	return args
end

-- Dynamically create the select boxes for each rank in the guild using the rank name as the name.
-- @param useCache uses a cached copy of the guild information if true, otherwise rebuilds it
-- @usage CreateRanks(true)
-- @usage CreateRanks()
-- @return nil
function Guilder:CreateRanks(useCache)
	if IsInGuild() then
		-- Get rank names
		local r = {}
		for i = 1, GuildControlGetNumRanks() do
			local rankName
			-- if useCache = true then used the cached ranks, else recreate them
			if useCache then
				rankName = dbCache.ranknames[i]
			else
				rankName = GuildControlGetRankName(i)
				if rankName == nil or rankName == "" then rankName = "Rank "..i end
				if dbCache.ranknames == nil then
					dbCache.ranknames = {}
				end
				dbCache.ranknames[i] = rankName
			end
			r["rank"..i] = {
			  	type = "select",
			  	name = rankName,
				desc = AL["Set the rank type for this rank"],
				order = i,
				values = { ["1-Officer"] = AL["Officer"], ["2-OffAlt"] = AL["Officer Alt"], ["3-Member"] = AL["Member"], ["4-Alt"] = AL["Alt"] },
				get = function() return Guilder.db.profile.dbCache[myNameFaction].ranks[i] end,
				set = function(info, v)
					Guilder.db.profile.dbCache[myNameFaction].ranks[i] = v
					Guilder:UpdateText()
				end,
			}
		end
		-- place rank names in the config screen
		options.args.defaultArgs.args.ranks.args = r
	else
		options.args.defaultArgs.args.ranks.args = {
			info = {
				type = "description",
				name = AL["You are not in a guild!"],
			},
		}
	end
end

--[[ *** CATCH EVENTS *** ]]--

function Guilder:WhisperHandler(...)
--[[
	Here we get whisper and filter out those the player should not see
]]
	if arg1 and arg1 ~= "" then
		-- Checks incoming whispers.
		msg = arg1
		if (string.find(msg, "^guilder") and event == "CHAT_MSG_WHISPER") then
			return
		end
		-- This checks outgoing whispers.
		if (string.find(msg:lower(), "^%[guilder%]") and event == "CHAT_MSG_WHISPER_INFORM") then
			return
		end
	end
	return self.hooks["ChatFrame_MessageEventHandler"](...)
end

function Guilder:CHAT_MSG_WHISPER()
	local msg = self:GetArgs(arg1, " ")
	local sender = arg2
	-- Check if the sender issued a 'guilder' command.
	if (string.lower(msg[1]) ~= "guilder") and (string.lower(msg[1]) ~= "gldr") then
		if string.lower(msg[1]) == "[guilder]" then
			return true
		end
		if db.profile.modules["Busy Responder"] then
			self:GetModule("Busy Responder"):SendBusyResponder(sender, arg1)
		end
		return true
	end
	-- Process the 'guilder' command.
	local command
	if msg[2] then
		command = string.lower(msg[2])
	end
	-- Sender requested guilder command help.
	if command == nil or command == string.lower(AL["help"]) then
		self:SendCommandHelp(sender)
		return true
	end
	-- Sender requested guild information
	if command == string.lower(AL["guildinfo"]) then
		self:SendGuildInfo(sender)
		return true
	end
	-- Sender requested cctargets info.
	if command == string.lower(AL["cc"]) then
		if db.profile.modules["Crowd Control"] then
			self:GetModule("Crowd Control"):CCTargets("WHISPER", sender)
		else
			self:SendMessage(AL["Crowd control module is not enabled."], "WHISPER", sender)
		end
		return true
	end
	if command == string.lower(AL["version"]) then
		if db.profile.modules["Version Check"] then
			self:SendMessage(AL["Guilder Version"]..": "..myVersionNumber, "WHISPER", sender)
		else
			self:SendMessage(AL["Version module is not enabled."], "WHISPER", sender)
		end
	end
	-- If we haven't returned yet, command must have been unrecognised.
	self:SendMessage(AL["Unrecognised command received"], "WHISPER", sender)
	self:SendMessage(AL["Whisper 'guilder help' for a list of valid commands"], "WHISPER", sender)
end

function Guilder:GUILD_ROSTER_UPDATE()
	-- test if the ranks have changed
	local ranksChanged = false
	for i = 1, 10 do
		if dbCache.ranknames[i] ~= GuildControlGetRankName(i) then ranksChanged = true end
	end
	if ranksChanged then self:CreateRanks() end
end

function Guilder:PLAYER_GUILD_UPDATE()
	if IsInGuild() then
		self:CreateRanks()
	else
		-- clear the cached ranks
		for i = 1, 10 do
			dbCache.ranknames[i] = "3-Member"
		end
	end
end

--[[ *** GUILDER FUNCTIONS *** ]]--

function Guilder:ExecuteChatCommand(arg)
--[[
	This function attempts to execute the supplied argument as though it were a command entered by
	the user in the chat window.
]]
	if (not DEFAULT_CHAT_FRAME) then return end
	DEFAULT_CHAT_FRAME.editBox:SetText(arg);
	ChatEdit_SendText(DEFAULT_CHAT_FRAME.editBox);
end

function Guilder:FindOfficerAlt(mainName)
--[[
	This function attempts to find the officer's name if they are on an alternate character. This
	assumes that the main character's name is in the Officers' Notes field.
	returns AltName (MainName)
]]
	if self:IsOfficer(mainName) and (not LG:IsMemberOnline(mainName)) then
		for name in LG:GetIterator("NAME", true) do
			oNote = LG:GetOfficerNote(name)
			if oNote ~= nil then
				if (LG:IsMemberOnline(name) and self:IsOfficerAlt(name) and string.match(oNote:upper(), mainName:upper())) then
					return name.." ("..mainName..")"
				end
			end
		end
	end
	return nil
end

function Guilder:GetNumberOfGuildOfficers()
--[[
	This function looks for characters holding the Officer rank and then simply performs a count to
	determine how many such characters there
]]
	local numOfficers = 0
	if IsInGuild() then
		for name in LG:GetIterator("NAME", true) do
			if self:IsOfficer(name) then numOfficers = numOfficers + 1 end
		end
	end
	return numOfficers
end

function Guilder:GetNumberOfGuildOfficersOnline()

--[[
	This function looks for characters holding the Officer rank and then simply performs a count to
	determine how many such characters there are online at the moment.
]]
	local numOfficers = 0
	if IsInGuild() then
		for name in LG:GetIterator("NAME", true) do
			if (self:IsOfficer(name) or self:IsOfficerAlt(name)) and LG:IsMemberOnline(name) then numOfficers = numOfficers + 1 end
		end
	end
	return numOfficers
end

function Guilder:IsOfficer(name)
--[[
	This function checks whether a character holds an officer rank
]]
	return dbCache.ranks[LG:GetRankIndex(name) + 1] == "1-Officer"
end

function Guilder:IsOfficerAlt(name)
--[[
	This function checks whether a character holds an officer's alt rank
]]
	return dbCache.ranks[LG:GetRankIndex(name) + 1] == "2-OffAlt"
end

function Guilder:OpenConfig(input)
--[[
	This function opens the Blizzard or the Ace3 configuration window
]]
	if input == "config" or not InterfaceOptionsFrame:IsResizable() then
		options.args.defaultArgs.guiHidden = true
		InterfaceOptionsFrame:Hide()
		ACD3:SetDefaultSize("Guilder", 500, 550)
		ACD3:Open("Guilder")
	else
		InterfaceOptionsFrame_OpenToCategory(self.optionsFrame.Profiles)
		options.args.defaultArgs.guiHidden = false
		InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
	end
end

function Guilder:SendCommandHelp(recipient)
--[[
	This function sends hints via whispers on the available whisper-based commands currently
	recognised by the mod to the named recipient.
]]
	self:SendMessage(AL["Whisper 'guilder [option]' where option is one of..."], "WHISPER", recipient)
	self:SendMessage(AL["[help] - displays this help"], "WHISPER", recipient)
	self:SendMessage(AL["[guildinfo] - displays more information about the guild"], "WHISPER", recipient)
	self:SendMessage(AL["Active modules"], "WHISPER", recipient)
	local moduleCount = 0
	if db.profile.modules["Crowd Control"] then
		self:SendMessage(AL["[cc] - whispers you the current Crowd Control targets"], "WHISPER", recipient)
		moduleCount = moduleCount + 1
	end
	if db.profile.modules["Version Check"] then
		self:SendMessage(AL["[version] - find out what version of Guilder the person is using"], "WHISPER", recipient)
		moduleCount = moduleCount + 1
	end
	if moduleCount == 0 then
		self:SendMessage(AL["No active modules found"], "WHISPER", recipient)
	end
end

function Guilder:SendGuildInfo(recipient)
--[[
	This function sends a summary of guild information via the whisper channel to the named
	recipient.
]]
	self:SendMessage(AL["guildinfo1"], "WHISPER", recipient)
	self:SendMessage(AL["guildinfo2"], "WHISPER", recipient)
	self:SendMessage(AL["guildinfo3"], "WHISPER", recipient)
	self:SendMessage(AL["guildinfo4"], "WHISPER", recipient)
	self:SendMessage(AL["guildinfo5"], "WHISPER", recipient)
end

function Guilder:SendMessage(messageText, channelName, recipient)
--[[
	This function is ostensibly a wrapper to the normal SendChatMessage function with minor
	changes as follows:
		1: Language is checked because it may not be passed in.
		2: If channelName = "SELF", output is routed to Guilder:Print() instead.
		3: If channelName = "NULL", output is muted.
	The intention for the existence of the NULL option for the channel is to allow for functions
	to pass a parameter that can be switched between a normal channel and null to turn on/off
	debug mode features.
]]
	-- Check whether the language option has been provided.
	if recipient == nil or recipient == " " then
		recipient = nil
	end

	if channelName == "WHISPER" and recipient == nil then
		self:Print("Error! Message sent with no recipient")
		return
	end

	-- Send chat output to 'chat' or to intended channel if not 'NULL'.
	if channelName == "SELF" then
		self:Print(messageText)
	elseif channelName ~= "NULL" then
		if channelName == "WHISPER" then messageText = "[Guilder] "..messageText end
		CTL:SendChatMessage("BULK", "", messageText, channelName, nil, recipient)
	end
end

function Guilder:UnitInFriends(characterName)
--[[
	This function determines whether the named character is a friend of the person running the mod.
]]
	local numFriends = GetNumFriends()
	for i = 1, numFriends do
		local name, level, class, zone, online, status, note = GetFriendInfo(i)
		if online and name == characterName then
			return true
		end
	end
	return false
end

function Guilder:UpdateConfig()
	for k, v in self:IterateModules() do
		if v:IsEnabled() then
			v:Disable()
			v:Enable()
		end
	end
end

function Guilder:UpdateText()
--[[
	Updates the LDB Text
]]
	if IsInGuild() then
		if db.profile.info.showOnline or db.profile.info.showTotal then
			local temptext = ""
			local needslash = false
			if db.profile.info.showOnline then
				temptext = temptext..self:GetNumberOfGuildOfficersOnline()
				needslash = true
			end
			if db.profile.info.showTotal then
				if needslash then temptext = temptext.."/" end
				temptext = temptext..self:GetNumberOfGuildOfficers()
			end
			if db.profile.info.showOnline then
				temptext = temptext.." "..AL["Online"]
			else
				temptext = temptext.." "..AL["Officers"]
			end
			LDBObj.text = temptext
		else
			LDBObj.text = "Guilder"
		end
	else
		if db.profile.info.showOnline or db.profile.info.showTotal then
			LDBObj.text = AL["No Guild"]
		else
			LDBObj.text = "Guilder"
		end
	end
end

--[[ *** LIBDATABROKER EVENT FUNCTIONS *** ]]--

function LDBObj:OnClick(button, modifier)
	if button == "LeftButton" then
		if IsAltKeyDown() and IsControlKeyDown() then
			-- Do nothing.
		elseif IsAltKeyDown() then
			Guilder:ExecuteChatCommand("/rbs report")
			return
		elseif IsControlKeyDown() then
			Guilder.db.profile.modules["Busy Responder"] = not Guilder.db.profile.modules["Busy Responder"]
			if Guilder.db.profile.modules["Busy Responder"] then
				Guilder:EnableModule("Busy Responder")
				Guilder:Print(AL["Enabled"], "Busy Responder", AL["Module"])
			else
				Guilder:DisableModule("Busy Responder")
				Guilder:Print(AL["Disabled"], "Busy Responder", AL["Module"])
			end
			return
		end
		Guilder:ExecuteChatCommand("/rbs toggle")
	elseif button == "RightButton" then
		Guilder:OpenConfig()
	end
end

function LDBObj:OnEnter()
	-- define header font size and color
	local headerFont = CreateFont("gldrHeaderFont")
	headerFont:SetFont(GameTooltipText:GetFont(), 18)
	headerFont:SetTextColor(1, 1, 0)
	-- create a QTip tooltip object (will be discarded after use)
	gldrTooltip = QTC:Acquire("GuilderTooltip", 1)
	gldrTooltip:Clear()
	gldrTooltip:SmartAnchorTo(self)
	-- modify tooltip to use Blizzard tooltip settings so that tooltip mods can modify it
	gldrTooltip:SetBackdrop(GameTooltip:GetBackdrop())
	gldrTooltip:SetBackdropBorderColor(GameTooltip:GetBackdropBorderColor())
	gldrTooltip:SetBackdropColor(GameTooltip:GetBackdropColor())
	gldrTooltip:SetScale(GameTooltip:GetScale())
	-- assign the header font
	gldrTooltip:SetHeaderFont(headerFont)
	
	self.tooltip = gldrTooltip
	-- populate the tooltip and display it
	LDBObj.ShowTooltip()
end

function LDBObj:OnLeave()
	gldrTooltip:Hide()
	QTC:Release(gldrTooltip) -- LibQTipClick
	gldrTooltip = nil
end

function LDBObj:ShowTooltip()
	gldrTooltip:AddHeader("Guilder")
	gldrTooltip:AddLine(" ")
	gldrTooltip:AddLine(WHITE..AL["Guild Officers"])
	for name in LG:GetIterator("NAME", true) do
		if Guilder:IsOfficer(name) then
			oAlt = Guilder:FindOfficerAlt(name)
			if LG:IsMemberOnline(name) then
				gldrTooltip:AddLine(GREEN..name)
			elseif oAlt ~= nil then
				gldrTooltip:AddLine(CYAN..oAlt)
			elseif Guilder.db.profile.info.showOffline then
				gldrTooltip:AddLine(RED..name)
			end
		end
	end
	if Guilder.db.profile.info.showModules then
	    gldrTooltip:AddLine(" ")
	    gldrTooltip:AddLine(WHITE..AL["Modules"])
	    local modules = {}
		for k, v in Guilder:IterateModules() do
			if v:IsEnabled() then
				table.insert(modules, k.." "..GREEN..AL["Enabled"])
			else
				table.insert(modules, k.." "..RED..AL["Disabled"])
			end
		end
		table.sort(modules)
		for i, v in ipairs(modules) do
			gldrTooltip:AddLine(v)
		end
	end
	if Guilder.db.profile.info.showHint then
	    gldrTooltip:AddLine(" ")
		gldrTooltip:AddLine(GREEN..AL["Hint"]..AL["TOOLTIP_HINT"])
	end
	gldrTooltip:Show()
end
