--[[ *** LOAD LIBRARIES *** ]]--
local Guilder = _G.Guilder
if not Guilder then
	error("Guilder_CrowdControl requires Guilder")
end

local Guilder_CrowdControl = Guilder:NewModule("CrowdControl")
local AL = Guilder.AL
Guilder_CrowdControl.modName = AL["Crowd Control"]

--[[ *** DEFAULT SETTINGS *** ]]--

local defaults = {
	profile = {
		channel = "SELF",
		skullName = "",
		skullType = "None",
		crossName = "",
		crossType = "None",
		squareName = "",
		squareType = "None",
		moonName = "",
		moonType = "None",
		triangleName = "",
		triangleType = "None",
		diamondName = "",
		diamondType = "None",
		circleName = "",
		circleType = "None",
		starName = "",
		starType = "None",
	}
}

--[[ *** CONFIGURATION MENU *** ]]--

local options = {
	Skull = {
		type = "group",
		name = AL["Skull"],
		desc = AL["Setup responsibilities for the Skull target"],
		order = 21,
		args = {
			name = {
				type = "input",
				name = AL["Set Name"],
				desc = AL["Set the name of the character responsible for this target"],
				order = 1,
				get = function() return Guilder_CrowdControl.db.profile.skullName end,
				set = function(info, v) Guilder_CrowdControl.db.profile.skullName = v end,
			},
			ttype = {
				type = "select",
				name = AL["Set Type"],
				desc = AL["Set the type of crowd control to be used on this target"],
				order = 2,
				values = {
					["Banish"] = AL["Banish"], ["Cyclone"] = AL["Cyclone"], ["Enslave"] = AL["Enslave"],
					["Fear"] = AL["Fear"], ["Hibernate"] = AL["Hibernate"], ["MT"] = AL["MT"],
					["MindControl"] = AL["Mind Control"], ["None"] = AL["None"], ["OT"] = AL["OT"],
					["Root"] = AL["Root"], ["Sap"] = AL["Sap"], ["Seduce"] = AL["Seduce"],
					["Shackle"] = AL["Shackle"], ["Sheep"] = AL["Sheep"], ["Trap"] = AL["Trap"],
					["TurnEvil"] = AL["Turn Evil"],
				},
				get = function() return Guilder_CrowdControl.db.profile.skullType end,
				set = function(info, v) Guilder_CrowdControl.db.profile.skullType = v end,
			},
			clear = {
				type = "execute",
				name = AL["Clear Name"],
				desc = AL["Clear this target of its assigned character and crowd control type"],
				order = 3,
				func = function()
					Guilder_CrowdControl.db.profile.skullType = AL["None"]
					Guilder_CrowdControl.db.profile.skullName = AL["EMPTY_STRING"]
				end,
			},
		},
	},
	Cross = {
		type = "group",
		name = AL["Cross"],
		desc = AL["Setup responsibilities for the Cross target"],
		order = 22,
		args = {
			name = {
				type = "input",
				name = AL["Set Name"],
				desc = AL["Set the name of the character responsible for this target"],
				order = 1,
				get = function() return Guilder_CrowdControl.db.profile.crossName end,
				set = function(info, v) Guilder_CrowdControl.db.profile.crossName = v end,
			},
			ttype = {
				type = "select",
				name = AL["Set Type"],
				desc = AL["Set the type of crowd control to be used on this target"],
				order = 2,
				values = {
					["Banish"] = AL["Banish"], ["Cyclone"] = AL["Cyclone"], ["Enslave"] = AL["Enslave"],
					["Fear"] = AL["Fear"], ["Hibernate"] = AL["Hibernate"], ["MT"] = AL["MT"],
					["MindControl"] = AL["Mind Control"], ["None"] = AL["None"], ["OT"] = AL["OT"],
					["Root"] = AL["Root"], ["Sap"] = AL["Sap"], ["Seduce"] = AL["Seduce"],
					["Shackle"] = AL["Shackle"], ["Sheep"] = AL["Sheep"], ["Trap"] = AL["Trap"],
					["TurnEvil"] = AL["Turn Evil"],
				},
				get = function() return Guilder_CrowdControl.db.profile.crossType end,
				set = function(info, v) Guilder_CrowdControl.db.profile.crossType = v end,
			},
			clear = {
				type = "execute",
				name = AL["Clear Name"],
				desc = AL["Clear this target of its assigned character and crowd control type"],
				order = 3,
				func = function()
					Guilder_CrowdControl.db.profile.crossType = AL["None"]
					Guilder_CrowdControl.db.profile.crossName = AL["EMPTY_STRING"]
				end,
			},
		},
	},
	Square = {
		type = "group",
		name = AL["Square"],
		desc = AL["Setup responsibilities for the Square target"],
		order = 23,
		args = {
			name = {
				type = "input",
				name = AL["Set Name"],
				desc = AL["Set the name of the character responsible for this target"],
				order = 1,
				get = function() return Guilder_CrowdControl.db.profile.squareName end,
				set = function(info, v) Guilder_CrowdControl.db.profile.squareName = v end,
			},
			ttype = {
				type = "select",
				name = AL["Set Type"],
				desc = AL["Set the type of crowd control to be used on this target"],
				order = 2,
				values = {
					["Banish"] = AL["Banish"], ["Cyclone"] = AL["Cyclone"], ["Enslave"] = AL["Enslave"],
					["Fear"] = AL["Fear"], ["Hibernate"] = AL["Hibernate"], ["MT"] = AL["MT"],
					["MindControl"] = AL["Mind Control"], ["None"] = AL["None"], ["OT"] = AL["OT"],
					["Root"] = AL["Root"], ["Sap"] = AL["Sap"], ["Seduce"] = AL["Seduce"],
					["Shackle"] = AL["Shackle"], ["Sheep"] = AL["Sheep"], ["Trap"] = AL["Trap"],
					["TurnEvil"] = AL["Turn Evil"],
				},
				get = function() return Guilder_CrowdControl.db.profile.squareType end,
				set = function(info, v) Guilder_CrowdControl.db.profile.squareType = v end,
			},
			clear = {
				type = "execute",
				name = AL["Clear Name"],
				desc = AL["Clear this target of its assigned character and crowd control type"],
				order = 3,
				func = function()
					Guilder_CrowdControl.db.profile.squareType = AL["None"]
					Guilder_CrowdControl.db.profile.squareName = AL["EMPTY_STRING"]
				end,
			},
		},
	},
	Moon = {
		type = "group",
		name = AL["Moon"],
		desc = AL["Setup responsibilities for the Moon target"],
		order = 24,
		args = {
			name = {
				type = "input",
				name = AL["Set Name"],
				desc = AL["Set the name of the character responsible for this target"],
				order = 1,
				get = function() return Guilder_CrowdControl.db.profile.moonName end,
				set = function(info, v) Guilder_CrowdControl.db.profile.moonName = v end,
			},
			ttype = {
				type = "select",
				name = AL["Set Type"],
				desc = AL["Set the type of crowd control to be used on this target"],
				order = 2,
				values = {
					["Banish"] = AL["Banish"], ["Cyclone"] = AL["Cyclone"], ["Enslave"] = AL["Enslave"],
					["Fear"] = AL["Fear"], ["Hibernate"] = AL["Hibernate"], ["MT"] = AL["MT"],
					["MindControl"] = AL["Mind Control"], ["None"] = AL["None"], ["OT"] = AL["OT"],
					["Root"] = AL["Root"], ["Sap"] = AL["Sap"], ["Seduce"] = AL["Seduce"],
					["Shackle"] = AL["Shackle"], ["Sheep"] = AL["Sheep"], ["Trap"] = AL["Trap"],
					["TurnEvil"] = AL["Turn Evil"],
				},
				get = function() return Guilder_CrowdControl.db.profile.moonType end,
				set = function(info, v) Guilder_CrowdControl.db.profile.moonType = v end,
			},
			clear = {
				type = "execute",
				name = AL["Clear Name"],
				desc = AL["Clear this target of its assigned character and crowd control type"],
				order = 3,
				func = function()
					Guilder_CrowdControl.db.profile.moonType = AL["None"]
					Guilder_CrowdControl.db.profile.moonName = AL["EMPTY_STRING"]
				end,
			},
		},
	},
	Triangle = {
		type = "group",
		name = AL["Triangle"],
		desc = AL["Setup responsibilities for the Triangle target"],
		order = 25,
		args = {
			name = {
				type = "input",
				name = AL["Set Name"],
				desc = AL["Set the name of the character responsible for this target"],
				order = 1,
				get = function() return Guilder_CrowdControl.db.profile.triangleName end,
				set = function(info, v) Guilder_CrowdControl.db.profile.triangleName = v end,
			},
			ttype = {
				type = "select",
				name = AL["Set Type"],
				desc = AL["Set the type of crowd control to be used on this target"],
				order = 2,
				values = {
					["Banish"] = AL["Banish"], ["Cyclone"] = AL["Cyclone"], ["Enslave"] = AL["Enslave"],
					["Fear"] = AL["Fear"], ["Hibernate"] = AL["Hibernate"], ["MT"] = AL["MT"],
					["MindControl"] = AL["Mind Control"], ["None"] = AL["None"], ["OT"] = AL["OT"],
					["Root"] = AL["Root"], ["Sap"] = AL["Sap"], ["Seduce"] = AL["Seduce"],
					["Shackle"] = AL["Shackle"], ["Sheep"] = AL["Sheep"], ["Trap"] = AL["Trap"],
					["TurnEvil"] = AL["Turn Evil"],
				},
				get = function() return Guilder_CrowdControl.db.profile.triangleType end,
				set = function(info, v) Guilder_CrowdControl.db.profile.triangleType = v end,
			},
			clear = {
				type = "execute",
				name = AL["Clear Name"],
				desc = AL["Clear this target of its assigned character and crowd control type"],
				order = 3,
				func = function()
					Guilder_CrowdControl.db.profile.triangleType = AL["None"]
					Guilder_CrowdControl.db.profile.triangleName = AL["EMPTY_STRING"]
				end,
			},
		},
	},
	Diamond = {
		type = "group",
		name = AL["Diamond"],
		desc = AL["Setup responsibilities for the Diamond target"],
		order = 26,
		args = {
			name = {
				type = "input",
				name = AL["Set Name"],
				desc = AL["Set the name of the character responsible for this target"],
				order = 1,
				get = function() return Guilder_CrowdControl.db.profile.diamondName end,
				set = function(info, v) Guilder_CrowdControl.db.profile.diamondName = v end,
			},
			ttype = {
				type = "select",
				name = AL["Set Type"],
				desc = AL["Set the type of crowd control to be used on this target"],
				order = 2,
				values = {
					["Banish"] = AL["Banish"], ["Cyclone"] = AL["Cyclone"], ["Enslave"] = AL["Enslave"],
					["Fear"] = AL["Fear"], ["Hibernate"] = AL["Hibernate"], ["MT"] = AL["MT"],
					["MindControl"] = AL["Mind Control"], ["None"] = AL["None"], ["OT"] = AL["OT"],
					["Root"] = AL["Root"], ["Sap"] = AL["Sap"], ["Seduce"] = AL["Seduce"],
					["Shackle"] = AL["Shackle"], ["Sheep"] = AL["Sheep"], ["Trap"] = AL["Trap"],
					["TurnEvil"] = AL["Turn Evil"],
				},
				get = function() return Guilder_CrowdControl.db.profile.diamondType end,
				set = function(info, v) Guilder_CrowdControl.db.profile.diamondType = v end,
			},
			clear = {
				type = "execute",
				name = AL["Clear Name"],
				desc = AL["Clear this target of its assigned character and crowd control type"],
				order = 3,
				func = function()
					Guilder_CrowdControl.db.profile.diamondType = AL["None"]
					Guilder_CrowdControl.db.profile.diamondName = AL["EMPTY_STRING"]
				end,
			},
		},
	},
	Circle = {
		type = "group",
		name = AL["Circle"],
		desc = AL["Setup responsibilities for the Circle target"],
		order = 27,
		args = {
			name = {
				type = "input",
				name = AL["Set Name"],
				desc = AL["Set the name of the character responsible for this target"],
				order = 1,
				get = function() return Guilder_CrowdControl.db.profile.circleName end,
				set = function(info, v) Guilder_CrowdControl.db.profile.circleName = v end,
			},
			ttype = {
				type = "select",
				name = AL["Set Type"],
				desc = AL["Set the type of crowd control to be used on this target"],
				order = 2,
				values = {
					["Banish"] = AL["Banish"], ["Cyclone"] = AL["Cyclone"], ["Enslave"] = AL["Enslave"],
					["Fear"] = AL["Fear"], ["Hibernate"] = AL["Hibernate"], ["MT"] = AL["MT"],
					["MindControl"] = AL["Mind Control"], ["None"] = AL["None"], ["OT"] = AL["OT"],
					["Root"] = AL["Root"], ["Sap"] = AL["Sap"], ["Seduce"] = AL["Seduce"],
					["Shackle"] = AL["Shackle"], ["Sheep"] = AL["Sheep"], ["Trap"] = AL["Trap"],
					["TurnEvil"] = AL["Turn Evil"],
				},
				get = function() return Guilder_CrowdControl.db.profile.circleType end,
				set = function(info, v) Guilder_CrowdControl.db.profile.circleType = v end,
			},
			clear = {
				type = "execute",
				name = AL["Clear Name"],
				desc = AL["Clear this target of its assigned character and crowd control type"],
				order = 3,
				func = function()
					Guilder_CrowdControl.db.profile.circleType = AL["None"]
					Guilder_CrowdControl.db.profile.circleName = AL["EMPTY_STRING"]
				end,
			},
		},
	},
	Star = {
		type = "group",
		name = AL["Star"],
		desc = AL["Setup responsibilities for the Star target"],
		order = 28,
		args = {
			name = {
				type = "input",
				name = AL["Set Name"],
				desc = AL["Set the name of the character responsible for this target"],
				order = 1,
				get = function() return Guilder_CrowdControl.db.profile.starName end,
				set = function(info, v) Guilder_CrowdControl.db.profile.starName = v end,
			},
			ttype = {
				type = "select",
				name = AL["Set Type"],
				desc = AL["Set the type of crowd control to be used on this target"],
				order = 2,
				values = {
					["Banish"] = AL["Banish"], ["Cyclone"] = AL["Cyclone"], ["Enslave"] = AL["Enslave"],
					["Fear"] = AL["Fear"], ["Hibernate"] = AL["Hibernate"], ["MT"] = AL["MT"],
					["MindControl"] = AL["Mind Control"], ["None"] = AL["None"], ["OT"] = AL["OT"],
					["Root"] = AL["Root"], ["Sap"] = AL["Sap"], ["Seduce"] = AL["Seduce"],
					["Shackle"] = AL["Shackle"], ["Sheep"] = AL["Sheep"], ["Trap"] = AL["Trap"],
					["TurnEvil"] = AL["Turn Evil"],
				},
				get = function() return Guilder_CrowdControl.db.profile.starType end,
				set = function(info, v) Guilder_CrowdControl.db.profile.starType = v end,
			},
			clear = {
				type = "execute",
				name = AL["Clear Name"],
				desc = AL["Clear this target of its assigned character and crowd control type"],
				order = 3,
				func = function()
					Guilder_CrowdControl.db.profile.starType = AL["None"]
					Guilder_CrowdControl.db.profile.starName = AL["EMPTY_STRING"]
				end,
			},
		},
	},
	Channel = {
		type = "select",
		name = AL["Set Channel"],
		desc = AL["Set the output channel for the crowd control announcement"],
		values = { ["RAID"] = AL["Raid"], ["PARTY"] = AL["Party"], ["OFFICER"] = AL["Officer"], ["GUILD"] = AL["Guild"], ["SAY"] = AL["Say"], ["SELF"] = AL["Self"] },
		order = 101,
		get = function() return Guilder_CrowdControl.db.profile.channel end,
		set = function(info, v) Guilder_CrowdControl.db.profile.channel = v end,
	},
	Announce = {
		type = "execute",
		name = AL["Announce"],
		desc = AL["Announce the crowd control targets to your current raid"],
		order = 102,
		func = function() Guilder_CrowdControl:CCTargets(Guilder_CrowdControl.db.profile.channel, nil) end,
	},
	ClearAll = {
		type = "execute",
		name = AL["Clear All"],
		desc = AL["Clear all targets of their assigned character and crowd control type"],
		order = 103,
		width = "full",
		func = function()
			Guilder_CrowdControl.db.profile.skullType = AL["None"]
			Guilder_CrowdControl.db.profile.skullName = AL["EMPTY_STRING"]
			Guilder_CrowdControl.db.profile.crossType = AL["None"]
			Guilder_CrowdControl.db.profile.crossName = AL["EMPTY_STRING"]
			Guilder_CrowdControl.db.profile.squareType = AL["None"]
			Guilder_CrowdControl.db.profile.squareName = AL["EMPTY_STRING"]
			Guilder_CrowdControl.db.profile.moonType = AL["None"]
			Guilder_CrowdControl.db.profile.moonName = AL["EMPTY_STRING"]
			Guilder_CrowdControl.db.profile.triangleType = AL["None"]
			Guilder_CrowdControl.db.profile.triangleName = AL["EMPTY_STRING"]
			Guilder_CrowdControl.db.profile.diamondType = AL["None"]
			Guilder_CrowdControl.db.profile.diamondName = AL["EMPTY_STRING"]
			Guilder_CrowdControl.db.profile.circleType = AL["None"]
			Guilder_CrowdControl.db.profile.circleName = AL["EMPTY_STRING"]
			Guilder_CrowdControl.db.profile.starType = AL["None"]
			Guilder_CrowdControl.db.profile.starName = AL["EMPTY_STRING"]
		end,
	},
}

--[[ *** MODULE FUNCTIONS *** ]]--

function Guilder_CrowdControl:OnInitialize()
	local db = Guilder.db:RegisterNamespace("Crowd Control", defaults)
	self.db = db
end

function Guilder_CrowdControl:OnEnable()
end

function Guilder_CrowdControl:OnDisable()
end

function Guilder_CrowdControl:Info()
	return AL["Set targets for crowd-control responsibilities with this tool"]
end

function Guilder_CrowdControl:GetOptions()
	return options
end

--[[ *** CROWD CONTROL FUNCTIONS *** ]]--

function Guilder_CrowdControl:CCTargets(channelName, sender)
--[[
	This function outputs an announcement in the channel specified by the channelName parameter
	(typically "RAID") to inform players of the crowd control target responsibilities that have
	been set up.
]]
	countNumberSetup = 0
	Guilder:SendMessage(AL["Crowd Control Responsibilities"], channelName, sender)
	if Guilder_CrowdControl.db.profile.skullType ~= AL["None"] then
		Guilder:SendMessage(AL["{skull}"].." - "..Guilder_CrowdControl.db.profile.skullType.." - "..Guilder_CrowdControl.db.profile.skullName, channelName, sender)
		countNumberSetup = countNumberSetup + 1
	end
	if Guilder_CrowdControl.db.profile.crossType ~= AL["None"] then
		Guilder:SendMessage(AL["{cross}"].." - "..Guilder_CrowdControl.db.profile.crossType.." - "..Guilder_CrowdControl.db.profile.crossName, channelName, sender)
		countNumberSetup = countNumberSetup + 1
	end
	if Guilder_CrowdControl.db.profile.squareType ~= AL["None"] then
		Guilder:SendMessage(AL["{square}"].." - "..Guilder_CrowdControl.db.profile.squareType.." - "..Guilder_CrowdControl.db.profile.squareName, channelName, sender)
		countNumberSetup = countNumberSetup + 1
	end
	if Guilder_CrowdControl.db.profile.moonType ~= AL["None"] then
		Guilder:SendMessage(AL["{moon}"].." - "..Guilder_CrowdControl.db.profile.moonType.." - "..Guilder_CrowdControl.db.profile.moonName, channelName, sender)
		countNumberSetup = countNumberSetup + 1
	end
	if Guilder_CrowdControl.db.profile.triangleType ~= AL["None"] then
		Guilder:SendMessage(AL["{triangle}"].." - "..Guilder_CrowdControl.db.profile.triangleType.." - "..Guilder_CrowdControl.db.profile.triangleName, channelName, sender)
		countNumberSetup = countNumberSetup + 1
	end
	if Guilder_CrowdControl.db.profile.diamondType ~= AL["None"] then
		Guilder:SendMessage(AL["{diamond}"].." - "..Guilder_CrowdControl.db.profile.diamondType.." - "..Guilder_CrowdControl.db.profile.diamondName, channelName, sender)
		countNumberSetup = countNumberSetup + 1
	end
	if Guilder_CrowdControl.db.profile.circleType ~= AL["None"] then
		Guilder:SendMessage(AL["{circle}"].." - "..Guilder_CrowdControl.db.profile.circleType.." - "..Guilder_CrowdControl.db.profile.circleName, channelName, sender)
		countNumberSetup = countNumberSetup + 1
	end
	if Guilder_CrowdControl.db.profile.starType ~= AL["None"] then
		Guilder:SendMessage(AL["{star}"].." - "..Guilder_CrowdControl.db.profile.starType.." - "..Guilder_CrowdControl.db.profile.starName, channelName, sender)
		countNumberSetup = countNumberSetup + 1
	end
	if countNumberSetup == 0 then
		Guilder:SendMessage(AL["No targets set up, expect imminent chaos!"], channelName, sender)
	end
end
